"""Generic logs pattern builder for Python and Flask, built on top of logging module."""
__author__ = "Ankur Mahajan"
__copyright__ = "Copyright 2023"
__version__ = "1.0.0"
__maintainer__ = "Ankur Mahajan"
__email__ = "mahajanankur.nith@gmail.com"

from motifer.logger_factory import LogFactory
from motifer.flask_factory import FlaskLogFactory